@@ -94,7 +94,7 @@ int connman_provider_indicate_error(struct connman_provider *provider,
 					enum connman_provider_error error);
 
 void connman_provider_set_index(struct connman_provider *provider, int index);
-int connman_provider_get_index(struct connman_provider *provider);
+int connman_provider_get_index(const struct connman_provider *provider);
 
 void connman_provider_set_data(struct connman_provider *provider, void *data);
 void *connman_provider_get_data(struct connman_provider *provider);
@@ -111,8 +111,12 @@ int connman_provider_set_domain(struct connman_provider *provider,
 				const char *domain);
 int connman_provider_set_nameservers(struct connman_provider *provider,
 					char * const *nameservers);
-int connman_provider_append_route(struct connman_provider *provider,
-					const char *key, const char *value);
+void connman_provider_set_autoconnect(struct connman_provider *provider,
+								bool flag);
+bool connman_provider_is_split_routing(struct connman_provider *provider);
+int connman_provider_set_split_routing(struct connman_provider *provider,
+							bool split_routing);
+int connman_provider_get_family(struct connman_provider *provider);
 
 const char *connman_provider_get_driver_name(struct connman_provider *provider);
 const char *connman_provider_get_save_group(struct connman_provider *provider);
