@@ -167,6 +167,9 @@ struct modem_data {
 	DBusPendingCall *call_get_contexts;
 };
 
+static int cm_get_contexts(struct modem_data *modem);
+static int cm_get_properties(struct modem_data *modem);
+
 static const char *api2string(enum ofono_api api)
 {
 	switch (api) {
@@ -289,7 +292,8 @@ static void set_connected(struct modem_data *modem,
 	char *nameservers;
 	int index;
 
-	DBG("%s", modem->path);
+	DBG("modem %p path %s context %p path %s",
+			modem, modem->path, context, context->path);
 
 	index = context->index;
 
@@ -300,10 +304,14 @@ static void set_connected(struct modem_data *modem,
 		return;
 	}
 
+	DBG("context->network %p", context->network);
+
 	service = connman_service_lookup_from_network(context->network);
 	if (!service)
 		return;
 
+	DBG("service %p", service);
+
 	connman_service_create_ip4config(service, index);
 	connman_network_set_ipv4_method(context->network, method);
 
@@ -1034,6 +1042,27 @@ static bool try_create_device(struct modem_data *modem)
 	modem->device = device;
 	connman_device_set_powered(modem->device, modem->online);
 
+	/*
+	 * The order in which the LTE and Connection Manager interfaces
+	 * arrive may not always be the same.
+	 *
+	 * If the latter arrived before the former and a modem device was
+	 * successfully created, this represents a secondary opportunity
+	 * to get Connection Manager properties and contexts, both
+	 * essential to successfully creating a Connection Manager
+	 * Cellular network and service.
+	 *
+	 * The primary opportunity would have been with api_added in
+	 * modem_update_interfaces; however, that would have been skipped
+	 * since the LTE interface did not yet exist in such a scenario
+	 * and this call would exit early due to the LTE_CAPABLE and
+	 * OFONO_API_LTE conditional checks above.
+	 */
+	if (has_interface(modem->interfaces, OFONO_API_CM)) {
+		cm_get_properties(modem);
+		cm_get_contexts(modem);
+	}
+
 	return true;
 }
 
@@ -1054,7 +1083,9 @@ static void add_network(struct modem_data *modem,
 {
 	const char *group;
 
-	DBG("%s", modem->path);
+	DBG("modem %p path %s context %p path %s network %p",
+		modem, modem->path,
+		context, context->path, context->network);
 
 	if (context->network)
 		return;
@@ -1147,6 +1178,35 @@ static int set_context_ipconfig(struct network_context *context,
 	return 0;
 }
 
+/**
+ *  @brief
+ *    Attempt to add an oFono context.
+ *
+ *  This evaluates the oFono context with the specified path and
+ *  dictionary. If it finds one of type 'internet', it adds the
+ *  context to the modem context list; adds the modem to the context
+ *  hash, keyed off the context path; and, if the context has a valid
+ *  APN, the modem is attached, and has a network registration
+ *  interface, a connman network object is created for the context.
+ *
+ *  @param[in,out]  modem         A pointer to the mutable modem data
+ *                                instance associated with @a
+ *                                context_path.
+ *  @param[in]      context_path  A pointer to an immutable, null-
+ *                                terminated C string containing the
+ *                                path of the context to evaluate.
+ *  @param[in]      dict          A pointer to a D-Bus message iterator
+ *                                for the dictionary associated with
+ *                                the context to evaluate.
+ *
+ *  @retval  0        If successful.
+ *  @retval  -ENOMEM  If memory could not be allocated for a context
+ *                    instance.
+ *  @retval  -EINVAL  If the context was not of type 'internet'.
+ *
+ *  @private
+ *
+ */
 static int add_cm_context(struct modem_data *modem, const char *context_path,
 				DBusMessageIter *dict)
 {
@@ -1154,12 +1214,15 @@ static int add_cm_context(struct modem_data *modem, const char *context_path,
 	struct network_context *context = NULL;
 	dbus_bool_t active = FALSE;
 	const char *ip_protocol = NULL;
+	int err = 0;
 
 	DBG("%s context path %s", modem->path, context_path);
 
 	context = network_context_alloc(context_path);
-	if (!context)
-		return -ENOMEM;
+	if (!context) {
+		err = -ENOMEM;
+		goto done;
+	}
 
 	while (dbus_message_iter_get_arg_type(dict) == DBUS_TYPE_DICT_ENTRY) {
 		DBusMessageIter entry, value;
@@ -1211,7 +1274,8 @@ static int add_cm_context(struct modem_data *modem, const char *context_path,
 
 	if (g_strcmp0(context_type, "internet") != 0) {
 		network_context_unref(context);
-		return -EINVAL;
+		err = -EINVAL;
+		goto done;
 	}
 
 	if (ip_protocol)
@@ -1226,7 +1290,10 @@ static int add_cm_context(struct modem_data *modem, const char *context_path,
 	    has_interface(modem->interfaces, OFONO_API_NETREG))
 		add_network(modem, context);
 
-	return 0;
+done:
+	DBG("err %d", err);
+
+	return err;
 }
 
 static void remove_cm_context(struct modem_data *modem,
@@ -1291,14 +1358,24 @@ static gboolean context_changed(DBusConnection *conn,
 
 	DBG("context_path %s", context_path);
 
+	/*
+	 * If there is no modem in the context hash for the associated
+	 * context path, then there is nothing to do at the moment as
+	 * the context must first be added before we can process changes
+	 * to it.
+	 */
 	modem = g_hash_table_lookup(context_hash, context_path);
 	if (!modem)
 		return TRUE;
 
+	DBG("modem %p", modem);
+
 	context = get_context_with_path(modem->context_list, context_path);
 	if (!context)
 		return TRUE;
 
+	DBG("context %p", context);
+
 	if (!dbus_message_iter_init(message, &iter))
 		return TRUE;
 
@@ -1371,6 +1448,8 @@ static gboolean context_changed(DBusConnection *conn,
 
 		dbus_message_iter_get_basic(&value, &ip_protocol);
 
+		DBG("%s Protocol %s", modem->path, ip_protocol);
+
 		set_context_ipconfig(context, ip_protocol);
 	}
 
@@ -1415,7 +1494,11 @@ static void cm_get_contexts_reply(DBusPendingCall *call, void *user_data)
 		dbus_message_iter_next(&entry);
 		dbus_message_iter_recurse(&entry, &value);
 
-		if (add_cm_context(modem, context_path, &value))
+		/*
+		 * If a context of type 'internet' is found, stop iterating;
+		 * we have the desired context.
+		 */
+		if (add_cm_context(modem, context_path, &value) == 0)
 			break;
 
 		dbus_message_iter_next(&dict);
@@ -1520,7 +1603,12 @@ static gboolean cm_context_removed(DBusConnection *conn,
 	if (!modem)
 		return TRUE;
 
+	DBG("modem %p", modem);
+
 	context = get_context_with_path(modem->context_list, context_path);
+
+	DBG("context %p", context);
+
 	remove_cm_context(modem, context);
 
 	return TRUE;
@@ -1772,10 +1860,14 @@ static gboolean cm_changed(DBusConnection *conn, DBusMessage *message,
 	DBusMessageIter iter, value;
 	const char *key;
 
+	DBG("");
+
 	modem = g_hash_table_lookup(modem_hash, path);
 	if (!modem)
 		return TRUE;
 
+	DBG("modem %p modem->ignore %u", modem, modem->ignore);
+
 	if (modem->ignore)
 		return TRUE;
 
@@ -1849,6 +1941,8 @@ static gboolean sim_changed(DBusConnection *conn, DBusMessage *message,
 	if (!modem)
 		return TRUE;
 
+	DBG("modem %p modem->ignore %u", modem, modem->ignore);
+
 	if (modem->ignore)
 		return TRUE;
 
@@ -1860,6 +1954,8 @@ static gboolean sim_changed(DBusConnection *conn, DBusMessage *message,
 	dbus_message_iter_next(&iter);
 	dbus_message_iter_recurse(&iter, &value);
 
+	DBG("key %s", key);
+
 	if (g_str_equal(key, "SubscriberIdentity")) {
 		sim_update_imsi(modem, &value);
 		try_create_device(modem);
@@ -1943,9 +2039,12 @@ static void modem_update_interfaces(struct modem_data *modem,
 				uint8_t old_ifaces,
 				uint8_t new_ifaces)
 {
-	DBG("%s", modem->path);
+	DBG("modem %p path %s", modem, modem->path);
 
 	if (api_added(old_ifaces, new_ifaces, OFONO_API_SIM)) {
+		DBG("modem->imsi %p modem->set_powered %u",
+				modem->imsi, modem->set_powered);
+
 		if (!modem->imsi &&
 				!modem->set_powered) {
 			/*
@@ -1957,6 +2056,8 @@ static void modem_update_interfaces(struct modem_data *modem,
 	}
 
 	if (api_added(old_ifaces, new_ifaces, OFONO_API_CM)) {
+		DBG("modem->device %p", modem->device);
+
 		if (modem->device) {
 			cm_get_properties(modem);
 			cm_get_contexts(modem);
