@@ -3,6 +3,7 @@
  *  Connection Manager
  *
  *  Copyright (C) 2012-2013  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2025  Jolla Mobile Ltd
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -945,7 +946,7 @@ static void do_dad(GDHCPClient *dhcp_client, struct connman_dhcpv6 *dhcp)
 
 		ref_own_address(user_data);
 
-		if (inet_pton(AF_INET6, address, &addr) < 0) {
+		if (inet_pton(AF_INET6, address, &addr) != 1) {
 			DBG("Invalid IPv6 address %s %d/%s", address,
 				-errno, strerror(errno));
 			goto fail;
@@ -2064,6 +2065,7 @@ static GDHCPClient *create_pd_client(struct connman_dhcpv6 *dhcp, int *err)
 
 	g_dhcpv6_client_create_iaid(dhcp_client, index, (unsigned char *)&iaid);
 	g_dhcpv6_client_set_iaid(dhcp_client, iaid);
+	*err = 0;
 
 	return dhcp_client;
 }
