@@ -42,8 +42,8 @@
 
 #include "connman.h"
 
-#ifndef ARPHDR_PHONET_PIPE
-#define ARPHDR_PHONET_PIPE (821)
+#ifndef ARPHRD_PHONET_PIPE
+#define ARPHRD_PHONET_PIPE (821)
 #endif
 
 #define print(arg...) do { if (0) connman_info(arg); } while (0)
@@ -448,7 +448,7 @@ static void process_newlink(unsigned short type, int index, unsigned flags,
 	switch (type) {
 	case ARPHRD_ETHER:
 	case ARPHRD_LOOPBACK:
-	case ARPHDR_PHONET_PIPE:
+	case ARPHRD_PHONET_PIPE:
 	case ARPHRD_PPP:
 	case ARPHRD_NONE:
 		__connman_ipconfig_newlink(index, type, flags,
@@ -536,7 +536,7 @@ static void process_dellink(unsigned short type, int index, unsigned flags,
 	switch (type) {
 	case ARPHRD_ETHER:
 	case ARPHRD_LOOPBACK:
-	case ARPHDR_PHONET_PIPE:
+	case ARPHRD_PHONET_PIPE:
 	case ARPHRD_PPP:
 	case ARPHRD_NONE:
 		__connman_ipconfig_dellink(index, &stats);
@@ -926,7 +926,7 @@ static inline void print_attr(struct rtattr *attr, const char *name)
 		print("  attr %d (len %d)\n", attr->rta_type, len);
 }
 
-static void rtnl_link(struct nlmsghdr *hdr, bool *has_master)
+static void rtnl_link(struct nlmsghdr *hdr)
 {
 	struct ifinfomsg *msg;
 	struct rtattr *attr;
@@ -968,7 +968,6 @@ static void rtnl_link(struct nlmsghdr *hdr, bool *has_master)
 			print_attr(attr, "priority");
 			break;
 		case IFLA_MASTER:
-			*has_master = true;
 			print_attr(attr, "master");
 			break;
 		case IFLA_WIRELESS:
@@ -1001,32 +1000,22 @@ static void rtnl_link(struct nlmsghdr *hdr, bool *has_master)
 
 static void rtnl_newlink(struct nlmsghdr *hdr)
 {
-	bool has_master = false;
 	struct ifinfomsg *msg = (struct ifinfomsg *) NLMSG_DATA(hdr);
 
-	rtnl_link(hdr, &has_master);
+	rtnl_link(hdr);
 
 	if (hdr->nlmsg_type == IFLA_WIRELESS)
 		connman_warn_once("Obsolete WEXT WiFi driver detected");
 
-	/* ignore RTM_NEWLINK when adding interface to bridge */
-	if (has_master)
-		return;
-
 	process_newlink(msg->ifi_type, msg->ifi_index, msg->ifi_flags,
 				msg->ifi_change, msg, IFA_PAYLOAD(hdr));
 }
 
 static void rtnl_dellink(struct nlmsghdr *hdr)
 {
-	bool has_master = false;
 	struct ifinfomsg *msg = (struct ifinfomsg *) NLMSG_DATA(hdr);
 
-	rtnl_link(hdr, &has_master);
-
-	/* ignore RTM_DELLINK when removing interface from bridge */
-	if (has_master)
-		return;
+	rtnl_link(hdr);
 
 	process_dellink(msg->ifi_type, msg->ifi_index, msg->ifi_flags,
 				msg->ifi_change, msg, IFA_PAYLOAD(hdr));
