@@ -1804,7 +1804,7 @@ static void session_activate(struct connman_session *session)
 		struct connman_service *service;
 		struct connman_service_info *info;
 		GSList *service_list = NULL;
-		enum connman_service_state state = CONNMAN_SESSION_STATE_DISCONNECTED;
+		enum connman_service_state state = CONNMAN_SERVICE_STATE_DISCONNECT;
 
 		g_hash_table_iter_init(&iter, service_hash);
 
@@ -1937,7 +1937,9 @@ static void service_state_changed(struct connman_service *service,
 {
 	struct connman_service_info *info;
 
-	DBG("service %p state %d", service, state);
+	DBG("service %p (%s) state %d",
+		service, connman_service_get_identifier(service),
+		state);
 
 	info = g_hash_table_lookup(service_hash, service);
 
