@@ -46,8 +46,8 @@
 
 #include "vpn-rtnl.h"
 
-#ifndef ARPHDR_PHONET_PIPE
-#define ARPHDR_PHONET_PIPE (821)
+#ifndef ARPHRD_PHONET_PIPE
+#define ARPHRD_PHONET_PIPE (821)
 #endif
 
 #define print(arg...) do { if (0) connman_info(arg); } while (0)
@@ -64,8 +64,6 @@ static GSList *watch_list = NULL;
 static unsigned int watch_id = 0;
 
 static GSList *update_list = NULL;
-static guint update_interval = G_MAXUINT;
-static guint update_timeout = 0;
 
 struct interface_data {
 	int index;
@@ -147,61 +145,6 @@ void vpn_rtnl_remove_watch(unsigned int id)
 	}
 }
 
-static void trigger_rtnl(int index, void *user_data)
-{
-	struct vpn_rtnl *rtnl = user_data;
-
-	if (rtnl->newlink) {
-		unsigned short type = __vpn_ipconfig_get_type_from_index(index);
-		unsigned int flags = __vpn_ipconfig_get_flags_from_index(index);
-
-		rtnl->newlink(type, index, flags, 0);
-	}
-}
-
-static GSList *rtnl_list = NULL;
-
-static gint compare_priority(gconstpointer a, gconstpointer b)
-{
-	const struct vpn_rtnl *rtnl1 = a;
-	const struct vpn_rtnl *rtnl2 = b;
-
-	return rtnl2->priority - rtnl1->priority;
-}
-
-/**
- * vpn_rtnl_register:
- * @rtnl: RTNL module
- *
- * Register a new RTNL module
- *
- * Returns: %0 on success
- */
-int vpn_rtnl_register(struct vpn_rtnl *rtnl)
-{
-	DBG("rtnl %p name %s", rtnl, rtnl->name);
-
-	rtnl_list = g_slist_insert_sorted(rtnl_list, rtnl,
-							compare_priority);
-
-	vpn_ipconfig_foreach(trigger_rtnl, rtnl);
-
-	return 0;
-}
-
-/**
- * vpn_rtnl_unregister:
- * @rtnl: RTNL module
- *
- * Remove a previously registered RTNL module
- */
-void vpn_rtnl_unregister(struct vpn_rtnl *rtnl)
-{
-	DBG("rtnl %p name %s", rtnl, rtnl->name);
-
-	rtnl_list = g_slist_remove(rtnl_list, rtnl);
-}
-
 static const char *operstate2str(unsigned char operstate)
 {
 	switch (operstate) {
@@ -296,7 +239,7 @@ static void process_newlink(unsigned short type, int index, unsigned flags,
 	switch (type) {
 	case ARPHRD_ETHER:
 	case ARPHRD_LOOPBACK:
-	case ARPHDR_PHONET_PIPE:
+	case ARPHRD_PHONET_PIPE:
 	case ARPHRD_NONE:
 		__vpn_ipconfig_newlink(index, type, flags,
 							str, mtu, &stats);
@@ -322,13 +265,6 @@ static void process_newlink(unsigned short type, int index, unsigned flags,
 					GINT_TO_POINTER(index), interface);
 	}
 
-	for (list = rtnl_list; list; list = list->next) {
-		struct vpn_rtnl *rtnl = list->data;
-
-		if (rtnl->newlink)
-			rtnl->newlink(type, index, flags, change);
-	}
-
 	for (list = watch_list; list; list = list->next) {
 		struct watch_data *watch = list->data;
 
@@ -346,7 +282,6 @@ static void process_dellink(unsigned short type, int index, unsigned flags,
 	struct rtnl_link_stats stats;
 	unsigned char operstate = 0xff;
 	const char *ifname = NULL;
-	GSList *list;
 
 	memset(&stats, 0, sizeof(stats));
 	extract_link(msg, bytes, NULL, &ifname, NULL, &operstate, &stats);
@@ -356,13 +291,6 @@ static void process_dellink(unsigned short type, int index, unsigned flags,
 						ifname, index, operstate,
 						operstate2str(operstate));
 
-	for (list = rtnl_list; list; list = list->next) {
-		struct vpn_rtnl *rtnl = list->data;
-
-		if (rtnl->dellink)
-			rtnl->dellink(type, index, flags, change);
-	}
-
 	switch (type) {
 	case ARPHRD_ETHER:
 	case ARPHRD_LOOPBACK:
@@ -428,7 +356,6 @@ static void extract_ipv6_route(struct rtmsg *msg, int bytes, int *index,
 static void process_newroute(unsigned char family, unsigned char scope,
 						struct rtmsg *msg, int bytes)
 {
-	GSList *list;
 	char dststr[INET6_ADDRSTRLEN], gatewaystr[INET6_ADDRSTRLEN];
 	int index = -1;
 
@@ -465,21 +392,12 @@ static void process_newroute(unsigned char family, unsigned char scope,
 
 		if (!IN6_IS_ADDR_UNSPECIFIED(&dst))
 			return;
-	} else
-		return;
-
-	for (list = rtnl_list; list; list = list->next) {
-		struct vpn_rtnl *rtnl = list->data;
-
-		if (rtnl->newgateway)
-			rtnl->newgateway(index, gatewaystr);
 	}
 }
 
 static void process_delroute(unsigned char family, unsigned char scope,
 						struct rtmsg *msg, int bytes)
 {
-	GSList *list;
 	char dststr[INET6_ADDRSTRLEN], gatewaystr[INET6_ADDRSTRLEN];
 	int index = -1;
 
@@ -516,14 +434,6 @@ static void process_delroute(unsigned char family, unsigned char scope,
 
 		if (!IN6_IS_ADDR_UNSPECIFIED(&dst))
 			return;
-	} else
-		return;
-
-	for (list = rtnl_list; list; list = list->next) {
-		struct vpn_rtnl *rtnl = list->data;
-
-		if (rtnl->delgateway)
-			rtnl->delgateway(index, gatewaystr);
 	}
 }
 
@@ -1017,78 +927,6 @@ static int send_getroute(void)
 	return queue_request(hdr);
 }
 
-static gboolean update_timeout_cb(gpointer user_data)
-{
-	__vpn_rtnl_request_update();
-
-	return TRUE;
-}
-
-static void update_interval_callback(guint min)
-{
-	if (update_timeout > 0)
-		g_source_remove(update_timeout);
-
-	if (min < G_MAXUINT) {
-		update_interval = min;
-		update_timeout = g_timeout_add_seconds(update_interval,
-						update_timeout_cb, NULL);
-	} else {
-		update_timeout = 0;
-		update_interval = G_MAXUINT;
-	}
-}
-
-static gint compare_interval(gconstpointer a, gconstpointer b)
-{
-	guint val_a = GPOINTER_TO_UINT(a);
-	guint val_b = GPOINTER_TO_UINT(b);
-
-	return val_a - val_b;
-}
-
-unsigned int __vpn_rtnl_update_interval_add(unsigned int interval)
-{
-	guint min;
-
-	if (interval == 0)
-		return 0;
-
-	update_list = g_slist_insert_sorted(update_list,
-			GUINT_TO_POINTER(interval), compare_interval);
-
-	min = GPOINTER_TO_UINT(g_slist_nth_data(update_list, 0));
-	if (min < update_interval) {
-		update_interval_callback(min);
-		__vpn_rtnl_request_update();
-	}
-
-	return update_interval;
-}
-
-unsigned int __vpn_rtnl_update_interval_remove(unsigned int interval)
-{
-	guint min = G_MAXUINT;
-
-	if (interval == 0)
-		return 0;
-
-	update_list = g_slist_remove(update_list, GINT_TO_POINTER(interval));
-
-	if (update_list)
-		min = GPOINTER_TO_UINT(g_slist_nth_data(update_list, 0));
-
-	if (min > update_interval)
-		update_interval_callback(min);
-
-	return min;
-}
-
-int __vpn_rtnl_request_update(void)
-{
-	return send_getlink();
-}
-
 int __vpn_rtnl_init(void)
 {
 	struct sockaddr_nl addr;
